﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using FileArrangement.App.Properties;
using Tomham.IO.FileManipulation;
using Tomham.Windows.Forms.AutoGeneration;

namespace FileArrangement.App
{
	public abstract class ComplexConditionController : Controller
	{
		private readonly ControllerCollection ControllersStore = new ControllerCollection();
		private string TitleTextStore = string.Empty;

		protected ComplexConditionController( object target, FormHelper formHelper ) : base( target )
		{
			FormHelper = formHelper;
		}

		private ComplexCondition CastedTarget
		{
			get { return (ComplexCondition) Target; }
		}

		protected string TitleText
		{
			get { return TitleTextStore; }
			set { TitleTextStore = value; }
		}

		private ControllerCollection Controllers
		{
			get { return ControllersStore; }
		}

		public FormHelper FormHelper { get; set; }

		private FlowLayoutPanel Root { get; set; }

		private FlowLayoutPanel ContentsFlowLayoutPanel { get; set; }

		public override void UpdateToControl()
		{
			RemoveContents();

			CreateAndAddContents();

			Controllers.UpdateToControlAll();
		}

		// todo: 不正な値を持つControlの色を変更するなどして目立たせる
		public override bool IsValidControlValues()
		{
			return Controllers.IsValidControllerValuesAll();
		}

		public override void UpdateToTarget()
		{
			CastedTarget.Clear();

			Controllers.UpdateToTargetAll();

			foreach ( Controller controller in Controllers )
			{
				CastedTarget.Add( (ICondition) controller.Target );
			}
		}

		protected override Control CreateControl()
		{
			SetUpControls();

			CreateAndAddContents();

			return Root;
		}

		private void CreateAndAddContents()
		{
			foreach ( ICondition condition in CastedTarget )
			{
				CreateAndAddContent( condition.Clone() );
			}
		}

		private void CreateAndAddContent( ICondition condition )
		{
			Control control = CreateContent( condition );

			ContentsFlowLayoutPanel.Controls.Add( control );
		}

		private void RemoveContents()
		{
			List<Control> olds = new List<Control>();

			foreach ( Control content in ContentsFlowLayoutPanel.Controls )
			{
				olds.Add( content );
			}

			foreach ( Control old in olds )
			{
				RemoveContent( old );
			}
		}

		private void RemoveContent( Control content )
		{
			Controller controller = GetController( content );

			Controllers.Remove( controller );

			ContentsFlowLayoutPanel.Controls.Remove( content );
			content.Dispose();
		}

		private Controller GetController( Control content )
		{
			foreach ( Controller controller in Controllers )
			{
				if ( content.Controls.Contains( controller.Control ) )
				{
					return controller;
				}
			}

			throw new InvalidOperationException();
		}

		private void SetUpControls()
		{
			ContentsFlowLayoutPanel = new FlowLayoutPanel
			                          {
			                          	BorderStyle = BorderStyle.FixedSingle,
			                          	FlowDirection = FlowDirection.TopDown,
			                          	WrapContents = false,
			                          	AutoScroll = true,
			                          	AutoSize = true,
			                          	MinimumSize = new Size( 720, 220 )
			                          };

			ToolStrip manipulationToolStrip = CreateManipulationToolStrip();

			Label titleLabel = new Label
			                   {
			                   	Text = TitleText,
			                   	BackColor = Color.White,
			                   	AutoSize = true
			                   };

			Root = new FlowLayoutPanel
			       {
			       	FlowDirection = FlowDirection.TopDown,
			       	WrapContents = false,
			       	AutoSize = true,
			       	AutoScroll = true,
			       	MinimumSize = new Size( 800, 300 ),
			       };

			Root.Controls.Add( titleLabel );
			Root.Controls.Add( manipulationToolStrip );
			Root.Controls.Add( ContentsFlowLayoutPanel );
		}

		private ToolStrip CreateManipulationToolStrip()
		{
			ToolStrip manipulationToolStrip = new ToolStrip();

			ToolStripButton addButton = new ToolStripButton
			                            {
			                            	Text = "追加",
			                            	Image = Resources.file_new_b.ToBitmap()
			                            };
			addButton.Click += AddButton_Click;

			ToolStripButton pasteButton = new ToolStripButton
			                              {
			                              	Text = "貼り付け",
			                              	Image = Resources.paste_bl_1_b.ToBitmap()
			                              };
			pasteButton.Click += PasteButton_Click;

			manipulationToolStrip.Items.Add( addButton );
			manipulationToolStrip.Items.Add( pasteButton );
			return manipulationToolStrip;
		}

		private Control CreateContent( ICondition condition )
		{
			FlowLayoutPanel panel = new FlowLayoutPanel
			                        {
			                        	BorderStyle = BorderStyle.FixedSingle,
			                        	AutoSize = true,
			                        	FlowDirection = FlowDirection.TopDown,
			                        	WrapContents = false,
			                        	Margin = new Padding( 6, 3, 6, 3 )
			                        };

			panel.Controls.Add( CretateContentmanipulationToolStrip() );

			Controller controller = FormHelper.Factory.Create( condition );
			panel.Controls.Add( controller.Control );
			Controllers.Add( controller );

			return panel;
		}

		private ToolStrip CretateContentmanipulationToolStrip()
		{
			ToolStrip contentmanipulationToolStrip = new ToolStrip();

			ToolStripButton copyToolStripButton = new ToolStripButton
			                                      {
			                                      	Text = "コピー",
			                                      	Image = Resources.copy_b.ToBitmap()
			                                      };
			copyToolStripButton.Click += copyToolStripButton_Click;

			ToolStripButton cutToolStripButton = new ToolStripButton
			                                     {
			                                     	Text = "切り取り",
			                                     	Image = Resources.cut_b.ToBitmap()
			                                     };
			cutToolStripButton.Click += cutToolStripButton_Click;

			ToolStripButton removeToolStripButton = new ToolStripButton
			                                        {
			                                        	Text = "削除",
			                                        	Image = Resources.delete_b.ToBitmap()
			                                        };
			removeToolStripButton.Click += removeToolStripButton_Click;

			contentmanipulationToolStrip.Items.Add( copyToolStripButton );
			contentmanipulationToolStrip.Items.Add( cutToolStripButton );
			contentmanipulationToolStrip.Items.Add( removeToolStripButton );
			return contentmanipulationToolStrip;
		}

		private void removeToolStripButton_Click( object sender, EventArgs e )
		{
			Control content = ( (ToolStripButton) sender ).Owner.Parent;

			RemoveContent( content );
		}

		private void cutToolStripButton_Click( object sender, EventArgs e )
		{
			Control content = ( (ToolStripButton) sender ).Owner.Parent;
			Controller controller = GetController( content );

			if ( !controller.IsValidControlValues() )
			{
				MessageBox.Show( "無効な値が含まれています。" );
				return;
			}

			Copy( controller );
			RemoveContent( content );
		}

		private void copyToolStripButton_Click( object sender, EventArgs e )
		{
			Control content = ( (ToolStripButton) sender ).Owner.Parent;
			Controller controller = GetController( content );

			if ( !controller.IsValidControlValues() )
			{
				MessageBox.Show( "無効な値が含まれています。" );
				return;
			}

			Copy( controller );
		}

		private void Copy( Controller controller )
		{
			controller.UpdateToTarget();

			ICondition condition = (ICondition) controller.Target;

			FormHelper.ClipBoard.Value = condition.Clone();
		}

		private void AddButton_Click( object sender, EventArgs e )
		{
			ICondition condition = FormHelper.ShowSelectConditionForm();

			if ( condition != null )
			{
				CreateAndAddContent( condition );
			}
		}

		private void PasteButton_Click( object sender, EventArgs e )
		{
			if ( FormHelper.ClipBoard.Value is ICondition )
			{
				ICondition condition = (ICondition) FormHelper.ClipBoard.Value;

				CreateAndAddContent( condition.Clone() );
			}
		}
	}
}