﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using System.Threading;
using System.Windows.Forms;
using AtMarkIT.Windows.Forms;
using FileArrangement.App.Properties;
using FileArrangement.Plugins;
using Tomham.IO;
using Yaowi.Common.Serialization;
using XmlDeSerializer = Yaowi.Common.Serialization.XmlDeserializer;

namespace FileArrangement.App
{
	public partial class MainForm : Form
	{
		private const string Extension = "." + HostHelper.RuleSetExtension;
		private const int MaxInformationTextLength = 30000000;
		private const string RuleSetFileType = "FileArrangementRuleSetFile";
		private readonly HostHelper HostHelperStore = new HostHelper();
		private readonly MessageBoard InformationBoardStore = new MessageBoard();
		private readonly List<RunResult> Results = new List<RunResult>();

		private readonly EditableItemInformationCollection<RuleSet> RuleSetInformationsStore =
			new EditableItemInformationCollection<RuleSet>();

		private FormHelper FormHelperStore;

		public MainForm()
		{
			InitializeComponent();
		}

		public EditableItemInformationCollection<RuleSet> RuleSetInformations
		{
			get { return RuleSetInformationsStore; }
		}

		public HostHelper HostHelper
		{
			get { return HostHelperStore; }
		}

		public FormHelper FormHelper
		{
			get { return FormHelperStore; }
		}

		public MessageBoard InformationBoard
		{
			get { return InformationBoardStore; }
		}

		private void MainForm_Load( object sender, EventArgs e )
		{
			SetUp();
		}

		private void SetUp()
		{
			InformationLabeledTextBox.TextBox.MaxLength = 30010000;
			HostHelper.SetUp();

			LoadRuleSets();

			SetUpRuleSetInformationsLabeledFilterListBox();

			FormHelperStore = new FormHelper( HostHelper );

			SetUpIcons();
			SetDockStyle();

			RuleSetInformationsLabeledFilterListBox.FilterListBox.SelectFirstItem();
		}

		private void SetUpIcons()
		{
			NewItemToolStripButton.Image = Resources.file_new_b.ToBitmap();
			EditItemToolStripButton.Image = Resources.file_pen_b.ToBitmap();
			CopyItemToolStripButton.Image = Resources.copy_b.ToBitmap();
			RemoveItemsToolStripButton.Image = Resources.delete_b.ToBitmap();
			PreviewToolStripButton.Image = Resources.note_l_g.ToBitmap();
			ExecuteToolStripButton.Image = Resources.note_l_b.ToBitmap();
			UndoToolStripButton.Image = Resources.undo_b.ToBitmap();
		}

		private void SetDockStyle()
		{
			foreach ( Control control in RootTableLayoutPanel.Controls )
			{
				control.Dock = DockStyle.Fill;
			}

			RuleSetInformationsLabeledFilterListBox.Dock = DockStyle.Fill;
			RootTableLayoutPanel.Dock = DockStyle.Fill;
		}

		private void SetUpRuleSetInformationsLabeledFilterListBox()
		{
			RuleSetInformationsLabeledFilterListBox.FilterListBox.SetSource( RuleSetInformations );

			RuleSetInformationsLabeledFilterListBox.FilterListBox.IsMatchedItem = IsMatchedItem;
		}

		private bool IsMatchedItem( object item )
		{
			return item.ToString().ToLower().Contains(
				RuleSetInformationsLabeledFilterListBox.FilterListBox.FilterText.ToLower() );
		}

		private void UpdateRuleSetInformationsView()
		{
			RuleSetInformationsLabeledFilterListBox.FilterListBox.UpdateMatchedItems();
		}

		private void LoadRuleSets()
		{
			XmlDeSerializer deserializer = HostHelper.GetXmlDeserializer();

			string searchPattern = "*." + HostHelper.RuleSetExtension;

			string[] fileNames =
				Directory.GetFiles( HostHelper.RuleSetsDirectory, searchPattern );

			foreach ( string fileName in fileNames )
			{
				try
				{
					RuleSet ruleSet = (RuleSet) deserializer.Deserialize( fileName );

					EditableItemInformation<RuleSet> information =
						new EditableItemInformation<RuleSet>( ruleSet ) { FileName = fileName };

					information.Item.Name = Path.GetFileNameWithoutExtension( fileName );

					RuleSetInformations.Add( information );
				}
				catch ( Exception e )
				{
					InformationBoard.AddMessages(
						string.Format( "読み込み失敗: {0}", PathUtils.DoubleQuotString( fileName ) ) );
					InformationBoard.AddMessages( e.Message );
				}
			}

			UpdateRuleSetInformationsView();
		}

		private void TearDown()
		{
			SaveRuleSets();

			HostHelper.TearDown();
		}

		private void SaveRuleSets()
		{
			XmlSerializer serializer = HostHelper.GetXmlSerializer();

			foreach ( EditableItemInformation<RuleSet> information in RuleSetInformations )
			{
				if ( information.Modified )
				{
					serializer.Serialize( information.Item, information.FileName );

					information.Modified = false;
				}
			}
		}

		private void MainForm_FormClosed( object sender, FormClosedEventArgs e )
		{
			TearDown();
		}

		private void InformationUpdateTimer_Tick( object sender, EventArgs e )
		{
			string[] messages = InformationBoard.GetAndClearMessages();

			foreach ( string message in messages )
			{
				if ( MaxInformationTextLength < InformationLabeledTextBox.TextBox.TextLength )
				{
					InformationLabeledTextBox.TextBox.Clear();
					InformationLabeledTextBox.TextBox.AppendText( "\r\n" + "インフォメーションをクリアしました" );
				}

				InformationLabeledTextBox.TextBox.AppendText( message + "\r\n" );
			}
		}

		private void NewItem()
		{
			string name = FormHelper.ShowNameInputBox( "新規作成" );

			if ( IsValidName( name ) )
			{
				string fileName = CreateRuleSetFileName( name );

				RuleSet ruleSet = new RuleSet { Name = name };

				EditableItemInformation<RuleSet> information =
					new EditableItemInformation<RuleSet>( ruleSet )
					{
						FileName = fileName,
						Modified = true
					};

				RuleSetInformations.Add( information );

				UpdateRuleSetInformationsView();
				ClearFilterText();
			}
		}

		private void CopyItem()
		{
			if ( !HasOnlyOneSelectedItem() )
			{
				return;
			}

			EditableItemInformation<RuleSet> selectedItem =
				RuleSetInformationsLabeledFilterListBox.FilterListBox.GetSelectedItem<EditableItemInformation<RuleSet>>();

			string name = FormHelper.ShowNameInputBox(
				string.Format( "{0}のコピー", selectedItem.Item.Name ) );

			if ( IsValidName( name ) )
			{
				string fileName = CreateRuleSetFileName( name );

				RuleSet ruleSet = selectedItem.Item.Clone();

				ruleSet.Name = name;

				EditableItemInformation<RuleSet> information =
					new EditableItemInformation<RuleSet>( ruleSet )
					{
						FileName = fileName,
						Modified = true
					};

				RuleSetInformations.Add( information );

				UpdateRuleSetInformationsView();
				ClearFilterText();
			}
		}

		private void ClearFilterText()
		{
			RuleSetInformationsLabeledFilterListBox.FilterListBox.ClearFilterText();
		}

		private string CreateRuleSetFileName( string name )
		{
			return PathUtils.Combine( HostHelper.RuleSetsDirectory, name + "." + HostHelper.RuleSetExtension );
		}

		private bool IsValidName( string name )
		{
			if ( name == string.Empty )
			{
				return false;
			}

			foreach ( EditableItemInformation<RuleSet> information in RuleSetInformations )
			{
				if ( name.ToUpper() ==
				     information.Item.Name.ToUpper() )
				{
					return false;
				}
			}

			return true;
		}

		private void RuleSetInformationsLabeledFilterListBox_FilterListBox_ListBoxDoubleClick( object sender, EventArgs e )
		{
			EditItem();
		}

		private void EditItem()
		{
			if ( !HasOnlyOneSelectedItem() )
			{
				return;
			}

			EditableItemInformation<RuleSet> selectedItem =
				RuleSetInformationsLabeledFilterListBox.FilterListBox.GetSelectedItem<EditableItemInformation<RuleSet>>();

			using ( RuleSetEditForm editForm = new RuleSetEditForm() )
			{
				editForm.Target = selectedItem.Item;
				editForm.FormHelper = FormHelper;
				editForm.HostHelper = HostHelper;

				if ( editForm.ShowDialog() ==
				     DialogResult.OK )
				{
					selectedItem.Modified = true;
				}
			}
		}

		private bool HasOnlyOneSelectedItem()
		{
			return RuleSetInformationsLabeledFilterListBox.FilterListBox.GetSelectedItemsCount() == 1;
		}

		private void RemoveItems()
		{
			List<EditableItemInformation<RuleSet>> removeTargets =
				RuleSetInformationsLabeledFilterListBox.FilterListBox.GetSelectedItems<EditableItemInformation<RuleSet>>();

			if ( 0 < removeTargets.Count )
			{
				if ( FormHelper.ShowConfirmBox( this, "選択したアイテムを削除してよろしいですか？" ) )
				{
					DoRemoveItems( removeTargets );
				}
			}
		}

		private void DoRemoveItems( IEnumerable<EditableItemInformation<RuleSet>> removeTargets )
		{
			foreach ( EditableItemInformation<RuleSet> target in removeTargets )
			{
				RuleSetInformations.Remove( target );

				try
				{
					File.Delete( target.FileName );
				}
				catch ( Exception e )
				{
					InformationBoard.AddMessages( e.Message );
				}
			}

			UpdateRuleSetInformationsView();
			ClearFilterText();
		}

		private void NewItemToolStripMenuItem_Click( object sender, EventArgs e )
		{
			NewItem();
		}

		private void RemoveItemsToolStripMenuItem_Click( object sender, EventArgs e )
		{
			RemoveItems();
		}

		private void EditItemToolStripMenuItem_Click( object sender, EventArgs e )
		{
			EditItem();
		}

		private void CopyItemToolStripMenuItem_Click( object sender, EventArgs e )
		{
			CopyItem();
		}

		private void ExecuteSelectedItemsToolStripMenuItem_Click( object sender, EventArgs e )
		{
			ExecuteSelectedItems();
		}

		private void ExecuteSelectedItems()
		{
			ExecuteSelectedItems( new MoveMode( new MessageBoardOutput( InformationBoard ) ) );
		}

		private void PreviewSelectedItemsToolStripMenuItem_Click( object sender, EventArgs e )
		{
			PreviewSelectedItems();
		}

		private void PreviewSelectedItems()
		{
			ExecuteSelectedItems( new PreviewMode( new MessageBoardOutput( InformationBoard ) ) );
		}

		private void ExecuteSelectedItems( IRuleSetRunnerMode mode )
		{
			List<EditableItemInformation<RuleSet>> targets =
				RuleSetInformationsLabeledFilterListBox.FilterListBox.GetSelectedItems<EditableItemInformation<RuleSet>>();

			if ( targets.Count == 0 )
			{
				InformationBoard.AddMessages( "ルールセットを1つ以上選択してから実行してください" );

				return;
			}

			RuleSetRunner runner = new RuleSetRunner( mode );

			WaitDialog dialog = new WaitDialog
			                    {
			                    	Owner = this,
			                    	MainMsg = "進行状況",
			                    	ProgressMax = targets.Count,
			                    	ProgressMin = 0,
			                    	ProgressStep = 1,
			                    	ProgressValue = 0
			                    };

			Enabled = false;

			try
			{
				dialog.Show();

				for ( int i = 0; i < targets.Count; i++ )
				{
					if ( dialog.IsAborting )
					{
						break;
					}

					RuleSet current = targets[i].Item;

					dialog.SubMsg = string.Format( "ルールセット: {0}", current.Name );

					Application.DoEvents();

					RunResult result = runner.Run( current );

					if ( !( mode is PreviewMode ) )
					{
						Results.Add( result );
					}

					InformationBoard.AddMessages( string.Empty );

					dialog.ProgressMsg = string.Format( "{0} / {1} ", i + 1, targets.Count );
					dialog.PerformStep();

					Application.DoEvents();
				}

				if ( dialog.DialogResult ==
				     DialogResult.Abort )
				{
					dialog.SubMsg = "処理を中断しました。";
				}
				else
				{
					dialog.SubMsg = "処理を完了しました。";
					dialog.ProgressMsg = string.Empty;
				}
				Application.DoEvents();
				Thread.Sleep( 100 );
			}
			catch ( Exception e )
			{
				InformationBoard.AddMessages( e.Message );
			}
			finally
			{
				Activate();
				dialog.Close();

				Enabled = true;
			}
		}

		private void ConditionListUpToolStripMenuItem_Click( object sender, EventArgs e )
		{
			FormHelper.ShowTypeInformations( HostHelper.ConditionTypeInformations, "条件一覧" );
		}

		private void EffectListUpToolStripMenuItem_Click( object sender, EventArgs e )
		{
			FormHelper.ShowTypeInformations( HostHelper.EffectTypeInformations, "効果一覧" );
		}

		private void ExitToolStripMenuItem_Click( object sender, EventArgs e )
		{
			Application.Exit();
		}

		private void RegisterExtensionToolStripMenuItem_Click( object sender, EventArgs e )
		{
			try
			{
				Assembly assembly = Assembly.GetExecutingAssembly();

				DirectoryInfo info = Directory.GetParent( assembly.Location );

				string path = PathUtils.Combine( info.FullName, "FileArrangementCui.exe" );

				FileUtils.RegisterExtension(
					Extension,
					path,
					RuleSetFileType,
					string.Empty
					);

				InformationBoard.AddMessages( "関連付け完了" );
			}
			catch ( Exception exception )
			{
				InformationBoard.AddMessages( "関連付けの失敗" );
				InformationBoard.AddMessages( exception.Message );
			}
		}

		private void UnRegisterExtensionToolStripMenuItem_Click( object sender, EventArgs e )
		{
			try
			{
				FileUtils.UnRegisterExtension(
					Extension,
					RuleSetFileType
					);

				InformationBoard.AddMessages( "関連付けの解除" );
			}
			catch ( Exception exception )
			{
				InformationBoard.AddMessages( "関連付けの解除失敗" );
				InformationBoard.AddMessages( exception.Message );
			}
		}

		private void VersionToolStripMenuItem_Click( object sender, EventArgs e )
		{
			InformationBoard.AddMessages( "1.05" );
		}

		private void RuleSetInformationsLabeledFilterListBox_FilterListBox_ListBoxKeyDown( object sender, KeyEventArgs e )
		{
			if ( e.KeyCode == Keys.Enter &&
			     !e.Shift &&
			     !e.Control &&
			     !e.Alt )
			{
				EditItem();
			}
		}

		private void EditGeneralsToolStripMenuItem_Click( object sender, EventArgs e )
		{
			using ( EditForm form = new EditForm { Text = "General" } )
			{
				form.FormHelper = FormHelper;

				form.SetTarget( HostHelper.Generals );

				form.ShowDialog();
			}
		}

		private void NewItemToolStripButton_Click( object sender, EventArgs e )
		{
			NewItem();
		}

		private void CopyItemToolStripButton_Click( object sender, EventArgs e )
		{
			CopyItem();
		}

		private void EditItemToolStripButton_Click( object sender, EventArgs e )
		{
			EditItem();
		}

		private void RemoveItemsToolStripButton_Click( object sender, EventArgs e )
		{
			RemoveItems();
		}

		private void ExecuteToolStripButton_Click( object sender, EventArgs e )
		{
			ExecuteSelectedItems();
		}

		private void PreviewToolStripButton_Click( object sender, EventArgs e )
		{
			PreviewSelectedItems();
		}

		private void UndoToolStripMenuItem_Click( object sender, EventArgs e )
		{
			Undo();
		}

		private void Undo()
		{
			if ( Results.Count == 0 )
			{
				InformationBoard.AddMessages( "元に戻す物がありません" );
				return;
			}

			Enabled = false;

			WaitDialog dialog = new WaitDialog
			                    {
			                    	Owner = this,
			                    	MainMsg = "進行状況",
			                    	ProgressMax = 1,
			                    	ProgressMin = 0,
			                    	ProgressStep = 1,
			                    	ProgressValue = 0
			                    };

			try
			{
				dialog.Show();

				InformationBoard.AddMessages( "それなりに元に戻す開始" );

				for ( int i = Results.Count - 1; 0 <= i; i-- )
				{
					Undo( Results[i] );
				}

				Results.Clear();

				InformationBoard.AddMessages( "それなりに元に戻す終了" );
				InformationBoard.AddMessages( string.Empty );
			}
			catch ( Exception e )
			{
				InformationBoard.AddMessages( e.Message );
			}
			finally
			{
				dialog.Close();
				Enabled = true;
				Activate();
			}
		}

		private void Undo( RunResult runResult )
		{
			for ( int i = runResult.ProcessResults.Count - 1; 0 <= i; i-- )
			{
				ProcessResult processResult = runResult.ProcessResults[i];

				try
				{
					CreateAncestry( processResult.Source );

					FileUtils.ForceMoveFile( processResult.Destination, processResult.Source );
					InformationBoard.AddMessages(
						string.Format( "{0} -> {1}",
						               PathUtils.DoubleQuotString( processResult.Destination ),
						               PathUtils.DoubleQuotString( processResult.Source ) ) );
				}
				catch ( Exception e )
				{
					InformationBoard.AddMessages(
						string.Format( "ファイル移動に失敗: {0} -> {1}",
						               PathUtils.DoubleQuotString( processResult.Destination ),
						               PathUtils.DoubleQuotString( processResult.Source ) ) );
					InformationBoard.AddMessages( e.Message );
				}
			}

			for ( int i = runResult.CreatedDirectories.Count - 1; 0 <= i; i-- )
			{
				string directory = runResult.CreatedDirectories[i];

				try
				{
					Directory.Delete( directory );
					InformationBoard.AddMessages(
						string.Format( "ディレクトリの削除: {0}", PathUtils.DoubleQuotString( directory ) ) );
				}
				catch ( Exception e )
				{
					InformationBoard.AddMessages(
						string.Format( "ディレクトリの削除に失敗: {0}", PathUtils.DoubleQuotString( directory ) ) );
					InformationBoard.AddMessages( e.Message );
				}
			}
		}

		private void CreateAncestry( string fileName )
		{
			string parent = PathUtils.GetParent( fileName );

			if ( !Directory.Exists( parent ) )
			{
				IList<string> unExistsDirectories = PathUtils.GetUnExistsDirectories( parent );

				foreach ( string directory in unExistsDirectories )
				{
					Directory.CreateDirectory( directory );

					string message =
						string.Format( "ディレクトリの作成: {0}", PathUtils.DoubleQuotString( directory ) );
					InformationBoard.AddMessages( message );
				}
			}
		}

		private void UndoToolStripButton_Click( object sender, EventArgs e )
		{
			Undo();
		}

		private void RuleSetInformationsLabeledFilterListBox_FilterListBox_FilterTextChanged(object sender, EventArgs e)
		{
			RuleSetInformationsLabeledFilterListBox.FilterListBox.SelectFirstItem();
		}
	}
}