﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using FileArrangement.App.Properties;
using FileArrangement.Plugins;
using Tomham.IO.FileManipulation;
using Tomham.Windows.Forms;

namespace FileArrangement.App
{
	public partial class RuleEditForm : Form
	{
		private Rule TargetStore;

		public RuleEditForm()
		{
			InitializeComponent();
		}

		public Rule Target
		{
			get { return TargetStore; }
			set
			{
				TargetStore = value;
				UpdateToControls();
			}
		}

		public HostHelper HostHelper { get; set; }
		public FormHelper FormHelper { get; set; }

		private ICondition Condition { get; set; }

		private void UpdateToControls()
		{
			NameLabeledTextBox.TextBox.Text = Target.Name;
			DestinationBrowseField.Text = Target.Destination;
			NewNamePatternLabeledTextBox.TextBox.Text = Target.NewNamePattern;

			Condition = Target.Condition.Clone();

			EffectsListBox.Items.Clear();

			foreach ( IEffect effect in Target.Effects )
			{
				EffectsListBox.Items.Add( new ListBoxItem( effect.Clone() ) );
			}
		}

		private void UpdateToTarget()
		{
			Target.Name = NameLabeledTextBox.TextBox.Text;
			Target.Destination = DestinationBrowseField.Text;
			Target.NewNamePattern = NewNamePatternLabeledTextBox.TextBox.Text;

			if ( Condition != null )
			{
				Target.Condition = Condition.Clone();
			}

			Target.Effects.Clear();

			foreach ( ListBoxItem item in EffectsListBox.Items )
			{
				IEffect effect = (IEffect) item.Value;

				Target.Effects.Add( effect.Clone() );
			}
		}

		private void okCancelPanel1_OKButton_Click( object sender, EventArgs e )
		{
			UpdateToTarget();

			DialogResult = DialogResult.OK;
		}

		private void okCancelPanel1_CancelButton_Click( object sender, EventArgs e )
		{
			DialogResult = DialogResult.Cancel;
		}

		private void RuleEditForm_Load( object sender, EventArgs e )
		{
			SetUpIcons();

			EffectsLabeledControl.SetContent( EffectsTableLayoutPanel );

			EffectsLabeledControl.Dock = DockStyle.Fill;
			EffectsTableLayoutPanel.Dock = DockStyle.Fill;
			EffectsListBox.Dock = DockStyle.Fill;

			NameLabeledTextBox.Dock = DockStyle.Fill;
			DestinationBrowseField.Dock = DockStyle.Fill;
			NewNamePatternLabeledTextBox.Dock = DockStyle.Fill;

			RootTableLayoutPanel.Dock = DockStyle.Fill;

			OKCancelPanel1.Dock = DockStyle.Fill;
		}

		private void SetUpIcons()
		{
			AddNewToolStripButton.Image = Resources.file_new_b.ToBitmap();
			CopyToolStripButton.Image = Resources.copy_b.ToBitmap();
			CutToolStripButton.Image = Resources.cut_b.ToBitmap();
			PasteToolStripButton.Image = Resources.paste_bl_1_b.ToBitmap();
			EditAllToolStripButton.Image = Resources.file_pen_b.ToBitmap();
			UpToolStripButton.Image = Resources.arrow_normal_up_b.ToBitmap();
			DownToolStripButton.Image = Resources.arrow_normal_down_b.ToBitmap();
			RemoveToolStripButton.Image = Resources.delete_b.ToBitmap();
		}

		private void ChangeConditionButton_Click( object sender, EventArgs e )
		{
			using ( ConditionEditForm editForm = new ConditionEditForm() )
			{
				editForm.FormHelper = FormHelper;
				editForm.HostHelper = HostHelper;

				if ( Condition != null )
				{
					editForm.SetCondition( Condition );
				}

				if ( editForm.ShowDialog() ==
				     DialogResult.OK )
				{
					Condition = editForm.Result;
				}
			}
		}

		private void UpToolStripButton_Click( object sender, EventArgs e )
		{
			EffectsListBox.MoveUpSelectedItems();
		}

		private void DownToolStripButton_Click( object sender, EventArgs e )
		{
			EffectsListBox.MoveDownSelectedItems();
		}

		private void EditAllToolStripButton_Click( object sender, EventArgs e )
		{
			EditEffects();
		}

		private void EditEffects()
		{
			if ( EffectsListBox.Items.Count == 0 )
			{
				return;
			}

			using ( EditForm form = new EditForm { Text = "効果の編集" } )
			{
				form.FormHelper = FormHelper;

				List<object> list = new List<object>();

				foreach ( ListBoxItem item in EffectsListBox.Items )
				{
					list.Add( item.Value );
				}

				form.SetTarget( list );

				form.ShowDialog();
			}
		}

		private void AddNewToolStripButton_Click( object sender, EventArgs e )
		{
			IEffect[] effects = FormHelper.ShowSelectEffectForm();

			foreach ( IEffect effect in effects )
			{
				EffectsListBox.Items.Add( new ListBoxItem( effect ) );
			}
		}

		private void RemoveToolStripButton_Click( object sender, EventArgs e )
		{
			Remove();
		}

		private void EffectsListBox_DoubleClick( object sender, EventArgs e )
		{
			EditEffects();
		}

		private void CopyToolStripButton_Click( object sender, EventArgs e )
		{
			if ( !HasOnlyOneSelectedItem() )
			{
				return;
			}

			Copy();
		}

		private void Copy()
		{
			ListBoxItem selectedItem = (ListBoxItem) EffectsListBox.SelectedItem;
			IEffect effect = (IEffect) selectedItem.Value;

			FormHelper.ClipBoard.Value = effect.Clone();
		}

		private void Remove()
		{
			EffectsListBox.RemoveSelectedItems();
		}

		private bool HasOnlyOneSelectedItem()
		{
			return EffectsListBox.SelectedItems.Count == 1;
		}

		private void PasteToolStripButton_Click( object sender, EventArgs e )
		{
			if ( FormHelper.ClipBoard.Value is IEffect )
			{
				IEffect effect = (IEffect) FormHelper.ClipBoard.Value;

				ListBoxItem item = new ListBoxItem( effect.Clone() );

				if ( 0 <= EffectsListBox.SelectedIndex )
				{
					EffectsListBox.Items.Insert( EffectsListBox.SelectedIndex, item );
				}
				else
				{
					EffectsListBox.Items.Add( item );
				}
			}
		}

		private void CutToolStripButton_Click( object sender, EventArgs e )
		{
			if ( !HasOnlyOneSelectedItem() )
			{
				return;
			}

			Copy();
			Remove();
		}

		private void EffectsListBox_KeyDown( object sender, KeyEventArgs e )
		{
			if ( e.KeyCode == Keys.Enter &&
			     !e.Shift &&
			     !e.Control &&
			     !e.Alt )
			{
				EditEffects();
			}
		}
	}
}