﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Windows.Forms;
using FileArrangement.App.Properties;
using Tomham.Windows.Forms;

namespace FileArrangement.App
{
	public partial class RuleSetEditForm : Form
	{
		private RuleSet TargetStore;

		public RuleSetEditForm()
		{
			InitializeComponent();
		}

		public RuleSet Target
		{
			get { return TargetStore; }
			set
			{
				TargetStore = value;
				UpdateToControls();
			}
		}

		public HostHelper HostHelper { get; set; }
		public FormHelper FormHelper { get; set; }

		private void UpdateToTarget()
		{
			if ( FilesRadioButton.Checked )
			{
				Target.TargetEntry = TargetEntry.Files;
			}
			else
			{
				Target.TargetEntry = TargetEntry.FilesRecursive;
			}

			Target.Rules.Clear();

			foreach ( Rule rule in RulesListBox.Items )
			{
				Target.Rules.Add( rule.Clone() );
			}

			Target.SourceDirectories.Clear();

			foreach ( string path in SourceDirectoriesBrowseList.Items )
			{
				if ( Directory.Exists( path ) )
				{
					Target.SourceDirectories.Add( path );
				}
			}
		}

		private void UpdateToControls()
		{
			Text = string.Format( "編集: ルールセット[{0}]", Target.Name );

			switch ( Target.TargetEntry )
			{
				case TargetEntry.Files:
					FilesRadioButton.Checked = true;
					break;

				case TargetEntry.FilesRecursive:
					FilesRecursiveRadioButton.Checked = true;
					break;
			}

			RulesListBox.Items.Clear();

			foreach ( Rule rule in Target.Rules )
			{
				RulesListBox.Items.Add( rule.Clone() );
			}

			SourceDirectoriesBrowseList.Items.Clear();

			foreach ( string source in Target.SourceDirectories )
			{
				SourceDirectoriesBrowseList.Items.Add( source );
			}
		}

		private void okCancelPanel1_OKButton_Click( object sender, EventArgs e )
		{
			UpdateToTarget();

			DialogResult = DialogResult.OK;
		}

		private void okCancelPanel1_CancelButton_Click( object sender, EventArgs e )
		{
			DialogResult = DialogResult.Cancel;
		}

		private void AddToolStripButton_Click( object sender, EventArgs e )
		{
			AddNewRule();
		}

		private void AddNewRule()
		{
			Rule rule = new Rule
			            {
			            	Name = "NewRule"
			            };

			RulesListBox.Items.Add( rule );
		}

		private void EditToolStripButton_Click( object sender, EventArgs e )
		{
			EditRule();
		}

		private void EditRule()
		{
			if ( !HasOnlyOneSelectedItem() )
			{
				return;
			}

			Rule selectedItem = (Rule) RulesListBox.SelectedItem;

			using ( RuleEditForm editForm = new RuleEditForm() )
			{
				editForm.FormHelper = FormHelper;
				editForm.HostHelper = HostHelper;
				editForm.Target = selectedItem;

				if ( editForm.ShowDialog() ==
				     DialogResult.OK )
				{
					UpdateRuleSetListBox();
					RulesListBox.Select();
				}
			}
		}

		private void UpdateRuleSetListBox()
		{
			int selectedIndex = RulesListBox.SelectedIndex;

			List<object> items = new List<object>();

			foreach ( object item in RulesListBox.Items )
			{
				items.Add( item );
			}

			RulesListBox.Items.Clear();

			RulesListBox.Items.AddRange( items.ToArray() );
			RulesListBox.SelectedIndex = selectedIndex;
		}

		private bool HasOnlyOneSelectedItem()
		{
			return RulesListBox.SelectedItems.Count == 1;
		}

		private void RuleSetEditForm_Load( object sender, EventArgs e )
		{
			RulesLabeledControl.SetContent( RulesTableLayoutPanel );

			SetUpIcons();

			RulesTableLayoutPanel.Dock = DockStyle.Fill;
			RulesListBox.Dock = DockStyle.Fill;

			RulesLabeledControl.Dock = DockStyle.Fill;
			SourceDirectoriesBrowseList.Dock = DockStyle.Fill;

			OKCancelPanel1.Dock = DockStyle.Fill;

			RootTableLayoutPanel.Dock = DockStyle.Fill;
		}

		private void SetUpIcons()
		{
			AddToolStripButton.Image = Resources.file_new_b.ToBitmap();
			CopyToolStripButton.Image = Resources.copy_b.ToBitmap();
			CutToolStripButton.Image = Resources.cut_b.ToBitmap();
			PasteToolStripButton.Image = Resources.paste_bl_1_b.ToBitmap();
			EditToolStripButton.Image = Resources.file_pen_b.ToBitmap();
			RemoveToolStripButton.Image = Resources.delete_b.ToBitmap();
			UpToolStripButton.Image = Resources.arrow_normal_up_b.ToBitmap();
			DownToolStripButton.Image = Resources.arrow_normal_down_b.ToBitmap();
		}

		private void DownToolStripButton_Click( object sender, EventArgs e )
		{
			RulesListBox.MoveDownSelectedItems();
		}

		private void UpToolStripButton_Click( object sender, EventArgs e )
		{
			RulesListBox.MoveUpSelectedItems();
		}

		private void RulesListBox_DoubleClick( object sender, EventArgs e )
		{
			EditRule();
		}

		private void RulesListBox_KeyDown( object sender, KeyEventArgs e )
		{
			if ( e.KeyCode == Keys.Enter &&
			     !e.Shift &&
			     !e.Control &&
			     !e.Alt )
			{
				EditRule();
			}
		}

		private void RemoveToolStripButton_Click( object sender, EventArgs e )
		{
			Remove();
		}

		private void Remove()
		{
			RulesListBox.RemoveSelectedItems();
		}

		private void CopyToolStripButton_Click( object sender, EventArgs e )
		{
			if ( !HasOnlyOneSelectedItem() )
			{
				return;
			}

			Copy();
		}

		private void Copy()
		{
			Rule selectedItem = (Rule) RulesListBox.SelectedItem;

			FormHelper.ClipBoard.Value = selectedItem.Clone();
		}

		private void PasteToolStripButton_Click( object sender, EventArgs e )
		{
			if ( FormHelper.ClipBoard.Value is Rule )
			{
				Rule rule = (Rule) FormHelper.ClipBoard.Value;

				if ( 0 <= RulesListBox.SelectedIndex )
				{
					RulesListBox.Items.Insert( RulesListBox.SelectedIndex, rule.Clone() );
				}
				else
				{
					RulesListBox.Items.Add( rule.Clone() );
				}
			}
		}

		private void CutToolStripButton_Click( object sender, EventArgs e )
		{
			if ( !HasOnlyOneSelectedItem() )
			{
				return;
			}

			Copy();
			Remove();
		}
	}
}