﻿using System;
using System.Collections.Generic;
using System.IO;
using Mozzarella.Utility;
using Tomham;
using Tomham.IO;
using XmlDeSerializer = Yaowi.Common.Serialization.XmlDeserializer;

namespace FileArrangement.CommandLine
{
	internal class Program
	{
		private static HostHelper HostHelper;

		private static void Main( string[] args )
		{
			try
			{
				SetUp();
				Work( args );
			}
			catch ( Exception err )
			{
				Console.WriteLine( err.Message );
				Console.WriteLine( "エラーにより中止しました" );
			}

			TearDown();
		}

		private static void TearDown()
		{
			HostHelper.TearDown();
		}

		private static void SetUp()
		{
			HostHelper = new HostHelper();
			HostHelper.SetUp();
		}

		private static void Work( string[] args )
		{
			Arguments commandLine = new Arguments( args );

			string[] ruleSetFiles = CreateRuleSetFileList( commandLine );

			RuleSet[] ruleSets = LoadRuleSets( ruleSetFiles );

			IRuleSetRunnerMode mode;

			if ( IsPreviewMode( commandLine ) )
			{
				mode = new PreviewMode( new ConsoleMessageOutput() );
			}
			else
			{
				mode = new MoveMode( new ConsoleMessageOutput() );
			}

			RuleSetRunner runner = new RuleSetRunner( mode );

			foreach ( RuleSet ruleSet in ruleSets )
			{
				runner.Run( ruleSet );

				Console.WriteLine();
			}
		}

		private static RuleSet[] LoadRuleSets( IEnumerable<string> ruleSetFiles )
		{
			XmlDeSerializer deserializer = HostHelper.GetXmlDeserializer();

			List<RuleSet> result = new List<RuleSet>();

			foreach ( string fileName in ruleSetFiles )
			{
				try
				{
					RuleSet ruleSet = (RuleSet) deserializer.Deserialize( fileName );

					result.Add( ruleSet );
				}
				catch ( Exception )
				{
					Console.WriteLine( string.Format( "読み込み失敗: {0}", fileName ) );
				}
			}

			return result.ToArray();
		}

		private static bool IsPreviewMode( Arguments commandLine )
		{
			if ( commandLine["p"] != null )
			{
				return true;
			}

			return false;
		}

		private static string[] CreateRuleSetFileList( Arguments commandLine )
		{
			List<string> result = new List<string>();

			foreach ( string path in commandLine.AnonParams )
			{
				if ( FileUtils.FileOrDirectoryExists( path ) )
				{
					if ( Directory.Exists( path ) )
					{
						AddRuleSetFilesToFileListFromDirectory( result, path );
					}
					else
					{
						AddRuleSetFileToFileList( result, path );
					}
				}
				else
				{
					Console.WriteLine( path + "が見つかりません" );
				}
			}

			if ( commandLine["all"] != null )
			{
				AddRuleSetFilesToFileListFromDirectory( result, HostHelper.RuleSetsDirectory );
			}

			return result.ToArray();
		}

		private static void AddRuleSetFileToFileList( ICollection<string> list, string path )
		{
			if ( !list.Contains( path ) )
			{
				list.Add( path );
			}
		}

		private static void AddRuleSetFilesToFileListFromDirectory( ICollection<string> list, string path )
		{
			string searchPattern = "*." + HostHelper.RuleSetExtension;

			string[] files = Directory.GetFiles( path, searchPattern, SearchOption.AllDirectories );

			foreach ( string file in files )
			{
				AddRuleSetFileToFileList( list, file );
			}
		}
	}
}