﻿using System;
using System.IO;
using System.Security.Cryptography;
using Tomham.IO;

namespace FileArrangement.Plugins.Common
{
	[ControllerSpecification( typeof ( SameFileNameWorkEffectController ) )]
	[Information(
		"同名ファイルの処理",
		"バイナリを比較し、リネームや処理元ファイルの削除、キャンセルなどができます",
		"tom_ham",
		"1.00"
		)]
	public class SameFileNameWorkEffect : IEffect
	{
		public WantedFileManipulation WantedFileManipulation { get; set; }
		public UnWantedFileManipulation UnWantedFileManipulation { get; set; }
		public bool HasCheckedLastWriteTime { get; set; }

		#region IEffect Members

		public void RunningWithRule( RunningWithRuleArguments arguments )
		{
		}

		public void RanWithRule( RanWithRuleArguments arguments )
		{
		}

		public void RunningWithFile( RunningWithFileArguments arguments )
		{
		}

		public void ConditionMatching( ConditionMatchingArguments arguments )
		{
			string destination = PathUtils.Combine( arguments.DestinationDirectory,
			                                   arguments.NewName );

			if ( !File.Exists( destination ) )
			{
				return;
			}

			if ( HasCheckedLastWriteTime )
			{
				if ( FileUtils.CompareLastWriteTime( arguments.Source, destination ) <= 0 )
				{
					RunWithUnWantedFile( arguments );
					return;
				}
			}

			if ( IsSameFile( arguments.Source, destination ) )
			{
				RunWithUnWantedFile( arguments );
			}
			else
			{
				RunWithWantedFile( arguments );
			}
		}

		public void RanWithFile( RanWithFileArguments arguments )
		{
		}

		public IEffect Clone()
		{
			return (IEffect) MemberwiseClone();
		}

		#endregion

		private bool IsSameFile( string source, string destination )
		{
			string sourceHash = CalculateHash( source );
			string destinationHash = CalculateHash( destination );

			return sourceHash == destinationHash;
		}

		private string CalculateHash( string fileName )
		{
			FileStream sourceStream = new FileStream(
				fileName, FileMode.Open, FileAccess.Read );

			string result;

			try
			{
				MD5CryptoServiceProvider md5 = new MD5CryptoServiceProvider();

				byte[] raw = md5.ComputeHash( sourceStream );

				result = BitConverter.ToString( raw ).ToLower().Replace( "-", "" );
			}
			finally
			{
				sourceStream.Close();
			}

			return result;
		}

		private void RunWithUnWantedFile( ConditionMatchingArguments arguments )
		{
			switch ( UnWantedFileManipulation )
			{
				case UnWantedFileManipulation.Cancel:

					arguments.Cancel = true;
					break;

				case UnWantedFileManipulation.DeleteSource:

					if ( !arguments.PreviewMode )
					{
						FileUtils.ForceDeleteFile( arguments.Source );
					}

					arguments.Output.WriteLine(
						string.Format( "不要ファイルの削除: \"{0}\"", arguments.Source ) );

					arguments.Cancel = true;
					break;
			}
		}

		private void RunWithWantedFile( ConditionMatchingArguments arguments )
		{
			switch ( WantedFileManipulation )
			{
				case WantedFileManipulation.ReName:

					arguments.NewName = CreateNewName( arguments.DestinationDirectory, arguments.NewName );
					break;

				case WantedFileManipulation.None:
					break;
			}
		}

		private string CreateNewName( string destinationDirectory, string baseName )
		{
			string name;
			string extension;

			int index = baseName.LastIndexOf( "." );
			if ( index == -1 )
			{
				extension = string.Empty;
				name = baseName;
			}
			else
			{
				extension = baseName.Substring( index );
				name = baseName.Substring( 0, index );
			}

			for ( int i = 0; i < int.MaxValue; i++ )
			{
				string result = string.Format( "{0}[{1}]{2}", name, i, extension );

				string path = PathUtils.Combine( destinationDirectory, result );

				if ( !File.Exists( path ) )
				{
					return result;
				}
			}

			throw new ApplicationException();
		}
	}
}