﻿using System.Collections.Generic;
using System.IO;
using System.Reflection;
using System.Xml.Serialization;
using FileArrangement.Plugins;
using Tomham.IO;
using Tomham.IO.FileManipulation;
using Yaowi.Common.Serialization;
using XmlDeSerializer = Yaowi.Common.Serialization.XmlDeserializer;
using XmlSerializer=Yaowi.Common.Serialization.XmlSerializer;

namespace FileArrangement
{
	public class HostHelper
	{
		public const string RuleSetExtension = "fars";
		private readonly List<TypeInformation> ConditionTypeInformationsStore = new List<TypeInformation>();
		private readonly List<TypeInformation> EffectTypeInformationsStore = new List<TypeInformation>();
		private readonly List<IGeneral> GeneralsStore = new List<IGeneral>();
		private readonly List<TypeInformation> GeneralTypeInformationsStore = new List<TypeInformation>();

		public string PluginsDirectory { get; private set; }

		public string RuleSetsDirectory { get; private set; }

		public List<IGeneral> Generals
		{
			get { return GeneralsStore; }
		}

		public List<TypeInformation> GeneralTypeInformations
		{
			get { return GeneralTypeInformationsStore; }
		}

		public List<TypeInformation> ConditionTypeInformations
		{
			get { return ConditionTypeInformationsStore; }
		}

		public List<TypeInformation> EffectTypeInformations
		{
			get { return EffectTypeInformationsStore; }
		}

		private void LoadAllGenerals()
		{
			foreach ( IGeneral general in Generals )
			{
				general.Load();
			}
		}

		private void UnLoadAllGenerals()
		{
			foreach ( IGeneral general in Generals )
			{
				general.UnLoad();
			}
		}

		public void SetUp()
		{
			SetUpDirectories();
			SetUpPlugins();
		}

		public void TearDown()
		{
			UnLoadAllGenerals();
		}

		private void SetUpDirectories()
		{
			PluginsDirectory = GetDirectory( "Plugins" );
			RuleSetsDirectory = GetDirectory( "RuleSets" );

			if ( !Directory.Exists( PluginsDirectory ) )
			{
				Directory.CreateDirectory( PluginsDirectory );
			}

			if ( ! Directory.Exists( RuleSetsDirectory ) )
			{
				Directory.CreateDirectory( RuleSetsDirectory );
			}
		}

		private string GetDirectory( string name )
		{
			Assembly assembly = Assembly.GetExecutingAssembly();

			DirectoryInfo info = Directory.GetParent( assembly.Location );

			return PathUtils.Combine( info.FullName, name );
		}

		public TypeInformation[] GetTypeInformations( string interfaceName )
		{
			return PluginHelper.GetTypeInformations( PluginsDirectory, interfaceName );
		}

		private void SetUpPlugins()
		{
			SetUpTypeInformations();

			SetUpGenerals();
		}

		private void SetUpGenerals()
		{
			foreach ( TypeInformation typeInformation in GeneralTypeInformations )
			{
				Generals.Add( (IGeneral) typeInformation.Create() );
			}

			LoadAllGenerals();
		}

		private void SetUpTypeInformations()
		{
			SetUpBuiltinTypeInoformations();

			ConditionTypeInformations.AddRange(
				GetTypeInformations( typeof ( ICondition ).FullName ) );

			EffectTypeInformations.AddRange(
				GetTypeInformations( typeof ( IEffect ).FullName ) );

			GeneralTypeInformations.AddRange(
				GetTypeInformations( typeof ( IGeneral ).FullName ) );
		}

		private void SetUpBuiltinTypeInoformations()
		{
			ConditionTypeInformations.Add(
				new TypeInformation(
					typeof ( AndCondition ),
					"And",
					"全ての条件が満たされると真になります。条件を設定しない場合は偽になります",
					"tom_ham",
					"1.00",
					"Builtin" ) );

			ConditionTypeInformations.Add(
				new TypeInformation(
					typeof ( OrCondition ),
					"Or",
					"いずれかの条件が満たされると真になります。条件を設定しない場合は偽になります",
					"tom_ham",
					"1.00",
					"Builtin" ) );

			ConditionTypeInformations.Add(
				new TypeInformation(
					typeof ( NotCondition ),
					"Not",
					"条件の真偽を逆にします",
					"tom_ham",
					"1.00",
					"Builtin" ) );

			ConditionTypeInformations.Add(
				new TypeInformation(
					typeof ( NormalTextCondition ),
					"文字列比較",
					"名前のみ、もしくはフルパスを対象にできます",
					"tom_ham",
					"1.00",
					"Builtin" ) );

			ConditionTypeInformations.Add(
				new TypeInformation(
					typeof ( RegExCondition ),
					"正規表現比較",
					"名前のみ、もしくはフルパスを対象にできます",
					"tom_ham",
					"1.00",
					"Builtin" ) );

			ConditionTypeInformations.Add(
				new TypeInformation(
					typeof ( TrueCondition ),
					"True",
					"常に真を返す",
					"tom_ham",
					"1.00",
					"Builtin" ) );

			ConditionTypeInformations.Add(
				new TypeInformation(
					typeof ( FalseCondition ),
					"False",
					"常に偽を返す",
					"tom_ham",
					"1.00",
					"Builtin" ) );

			GeneralTypeInformations.Add(
				new TypeInformation(
					typeof ( TextConditionControllerHelper ),
					"TextConditionControllerHelper",
					string.Empty,
					"tom_ham",
					"1.00",
					"Builtin" ) );
		}

		public XmlDeSerializer GetXmlDeserializer()
		{
			XmlDeserializer deserializer = new XmlDeserializer();

			string[] dlls =
				Directory.GetFiles( PluginsDirectory, "*.dll" );

			foreach ( string dll in dlls )
			{
				Assembly assembly = Assembly.LoadFrom( dll );

				deserializer.RegisterAssembly( assembly );
			}

			return deserializer;
		}

		public XmlSerializer GetXmlSerializer()
		{
			XmlSerializer serializer = new XmlSerializer
			                           {
			                           	SerializationIgnoredAttributeType = typeof ( XmlIgnoreAttribute ),
			                           	IgnoreSerializableAttribute = true,
			                           };

			return serializer;
		}
	}
}