﻿using System;
using System.Collections.Generic;
using System.IO;
using FileArrangement.Plugins;
using Tomham.IO;
using Tomham.IO.FileManipulation;

namespace FileArrangement
{
	public class RuleSetRunner
	{
		public RuleSetRunner( IRuleSetRunnerMode mode )
		{
			Mode = mode;
		}

		public IRuleSetRunnerMode Mode { get; set; }

		private RunResult Result { get; set; }

		public RunResult Run( RuleSet ruleSet )
		{
			ClearResult();

			Mode.Running( ruleSet );

			foreach ( string sourceDirectory in ruleSet.SourceDirectories )
			{
				DoRun( sourceDirectory, ruleSet );
			}

			Mode.Ran( ruleSet, Result );

			return Result;
		}

		private void DoRun( string sourceDirectory, RuleSet ruleSet )
		{
			if ( !Directory.Exists( sourceDirectory ) )
			{
				return;
			}

			foreach ( Rule rule in ruleSet.Rules )
			{
				try
				{
					string message =
						string.Format( "処理元ディレクトリ: {0},  ルール: {1} の実行開始",
						               PathUtils.DoubleQuotString( sourceDirectory ),
						               rule.Name );

					WriteLine( message );
					RunWithRule( rule, sourceDirectory, ruleSet.TargetEntry );
				}
				catch ( Exception e )
				{
					string message =
						string.Format( "処理元ディレクトリ: {0},  ルール: {1} の実行中にエラーが発生しました",
						               sourceDirectory,
						               rule.Name );

					string logMessage =
						string.Format( "ルールセット: {0}, 処理元ディレクトリ: {1},  ルール: {2} の実行中にエラーが発生しました",
						               ruleSet.Name,
						               PathUtils.DoubleQuotString( sourceDirectory ),
						               rule.Name );

					WriteErrorMessages( message, logMessage, e );
				}
				finally
				{
					string message =
						string.Format( "処理元ディレクトリ: {0},  ルール: {1} の実行終了",
						               PathUtils.DoubleQuotString( sourceDirectory ),
						               rule.Name );

					WriteLine( message );
				}
			}
		}

		private void RunWithRule( Rule rule, string sourceDirectory, TargetEntry targetEntry )
		{
			DoEffectsRunningWithRule( rule.Effects,
			                          rule.Name,
			                          sourceDirectory );

			DoRunWithRule( rule, sourceDirectory, targetEntry );

			DoEffectsRanWithRule( rule.Effects,
			                      rule.Name,
			                      sourceDirectory );
		}

		private void DoRunWithRule( Rule rule, string sourceDirectory, TargetEntry targetEntry )
		{
			RunArguments arguments = new RunArguments
			                         {
			                         	SourceDirectory = sourceDirectory,
			                         	Rule = rule,
			                         	Output = Mode.Output,
			                         	TargetEntry = targetEntry
			                         };

			if ( arguments.SourceDirectory == null ||
			     !Directory.Exists( arguments.SourceDirectory ) )
			{
				throw new ArgumentException( "arguments.SourceDirectory is invalid." );
			}

			if ( !arguments.Rule.IsValid() )
			{
				throw new ApplicationException( "arguments.Rule is invalid." );
			}

			arguments.Rule.Adjust();

			RunWithDirectory( arguments, arguments.SourceDirectory );
		}

		private void DoEffectsRunningWithRule( IEnumerable<IEffect> effects, string name, string sourceDirectory )
		{
			RunningWithRuleArguments arguments =
				new RunningWithRuleArguments( Mode.Output,
				                              Mode is PreviewMode,
				                              name,
				                              sourceDirectory );

			foreach ( IEffect effect in effects )
			{
				effect.RunningWithRule( arguments );
			}
		}

		private void DoEffectsRanWithRule( IEnumerable<IEffect> effects,
		                                   string name,
		                                   string sourceDirectory )
		{
			RanWithRuleArguments arguments =
				new RanWithRuleArguments( Mode.Output,
				                          Mode is PreviewMode,
				                          name,
				                          sourceDirectory );

			foreach ( IEffect effect in effects )
			{
				effect.RanWithRule( arguments );
			}
		}

		private void ClearResult()
		{
			Result = new RunResult();
		}

		private void RunWithDirectory( RunArguments arguments, string directoryPath )
		{
			string[] entries = Directory.GetFiles( directoryPath );

			foreach ( string source in entries )
			{
				try
				{
					RunWithFile( arguments, source );
				}
				catch ( Exception e )
				{
					string message = string.Format( "エラー: {0}", PathUtils.DoubleQuotString( source ) );
					string logMessage = string.Format( "Source: {0}", PathUtils.DoubleQuotString( source ) );
					WriteErrorMessages( message, logMessage, e );
				}
			}

			if ( !Directory.Exists( directoryPath ) )
			{
				return;
			}

			if ( arguments.TargetEntry ==
			     TargetEntry.FilesRecursive )
			{
				string[] subDirectories = Directory.GetDirectories( directoryPath );

				foreach ( string subDirectoryName in subDirectories )
				{
					try
					{
						RunWithDirectory( arguments, subDirectoryName );
					}
					catch ( Exception e )
					{
						string message = string.Format( "エラー: {0}", PathUtils.DoubleQuotString( subDirectoryName ) );
						string logMessage = string.Format( "SourceDirectory: {0}", PathUtils.DoubleQuotString( subDirectoryName ) );
						WriteErrorMessages( message, logMessage, e );
					}
				}
			}
		}

		private void RunWithFile( RunArguments arguments, string source )
		{
			if ( HasProcessed( source ) )
			{
				return;
			}

			bool cancel = false;

			string destinationDirectory = arguments.Rule.Destination;

			DoEffectsRunningWithFile( arguments.Rule.Effects,
			                          source,
			                          ref destinationDirectory,
			                          ref cancel );

			if ( cancel )
			{
				return;
			}

			string newName = GetNewName( source, arguments.Rule.NewNamePattern );

			DoEffectsConditionMatching( arguments.Rule.Effects,
			                            source,
			                            destinationDirectory,
			                            ref newName,
			                            ref cancel );

			if ( cancel )
			{
				return;
			}

			string destination = PathUtils.Combine( destinationDirectory, newName );

			MatchArguments matchArguments =
				new MatchArguments( source, destination );

			if ( !arguments.Rule.Condition.IsMatch( matchArguments ) )
			{
				return;
			}

			CreateDirectory( destinationDirectory );

			Mode.Process( source, destination );

			Result.ProcessResults.Add( new ProcessResult { Source = source, Destination = destination } );

			DoEffectsRanWithFile( arguments.Rule.Effects, source, destination );
		}

		private bool HasProcessed( string fileName )
		{
			foreach ( ProcessResult item in Result.ProcessResults )
			{
				if ( item.Source == fileName )
				{
					return true;
				}
			}

			return false;
		}

		private void CreateDirectory( string destinationDirectory )
		{
			if ( Directory.Exists( destinationDirectory  ))
			{
				return;
			}

			IList<string> unExistsDirectories = PathUtils.GetUnExistsDirectories( destinationDirectory );

			foreach ( string directory in unExistsDirectories )
			{
				if ( !Result.CreatedDirectories.Contains( directory ) )
				{
					DoCreateDirectory( directory );
				}
			}
		}

		private void DoCreateDirectory( string directory )
		{
			Mode.CreateDirectory( directory );

			Result.CreatedDirectories.Add( directory );
		}

		private string GetNewName( string source, string newNamePattern )
		{
			if ( newNamePattern.Trim() ==
			     string.Empty )
			{
				return PathUtils.GetName( source );
			}

			return newNamePattern;
		}

		private void DoEffectsRunningWithFile( IEnumerable<IEffect> effects,
		                                       string source,
		                                       ref string destinationDirectory,
		                                       ref bool cancel )
		{
			RunningWithFileArguments args =
				new RunningWithFileArguments( Mode.Output,
				                              Mode is PreviewMode,
				                              source,
				                              destinationDirectory );

			foreach ( IEffect effect in effects )
			{
				effect.RunningWithFile( args );

				if ( args.Cancel )
				{
					cancel = args.Cancel;
					return;
				}
			}

			destinationDirectory = args.DestinationDirectory;
		}

		private void DoEffectsConditionMatching( IEnumerable<IEffect> effects,
		                                         string source,
		                                         string destinationDirectory,
		                                         ref string newName,
		                                         ref bool cancel )
		{
			ConditionMatchingArguments conditionMatchingArguments =
				new ConditionMatchingArguments( Mode.Output,
				                                Mode is PreviewMode,
				                                source,
				                                destinationDirectory,
				                                newName );

			foreach ( IEffect effect in effects )
			{
				effect.ConditionMatching( conditionMatchingArguments );

				if ( conditionMatchingArguments.Cancel )
				{
					cancel = conditionMatchingArguments.Cancel;

					return;
				}
			}

			newName = conditionMatchingArguments.NewName;
		}

		private void DoEffectsRanWithFile( IEnumerable<IEffect> effects, string source, string destination )
		{
			RanWithFileArguments ranWithFileArguments =
				new RanWithFileArguments( Mode.Output,
				                          Mode is PreviewMode,
				                          source,
				                          destination );

			foreach ( IEffect effect in effects )
			{
				effect.RanWithFile( ranWithFileArguments );
			}
		}

		private void WriteLine( string message )
		{
			Mode.Output.WriteLine( message );
		}

		private void WriteErrorMessages( string message, string logMessage, Exception e )
		{
			WriteLine( message );
			WriteLine( "エラーメッセージ:" );
			WriteLine( e.Message );
			WriteLine( "スタックトレース:" );
			WriteLine( e.StackTrace );

			Mode.Log.Error( logMessage, e );
		}
	}
}