﻿using System;
using System.Collections.Generic;
using System.Reflection;
using System.Windows.Forms;

namespace Tomham.Windows.Forms.AutoGeneration
{
	public class DefaultController : Controller
	{
		private readonly Dictionary<Property, Property> PropertyBindings =
			new Dictionary<Property, Property>();

		public DefaultController( object target ) : base( target )
		{
		}

		private FlowLayoutPanel Root { get; set; }

		protected override Control CreateControl()
		{
			SetUpControls();

			UpdateToControl();

			return Root;
		}

		public override void UpdateToTarget()
		{
			foreach ( KeyValuePair<Property, Property> pair in PropertyBindings )
			{
				pair.Key.PropertyInfo.SetValue(
					pair.Key.Target,
					pair.Value.PropertyInfo.GetValue( pair.Value.Target, null ),
					null );
			}
		}

		public override void UpdateToControl()
		{
			foreach ( KeyValuePair<Property, Property> pair in PropertyBindings )
			{
				pair.Value.PropertyInfo.SetValue(
					pair.Value.Target,
					pair.Key.PropertyInfo.GetValue( pair.Key.Target, null ),
					null );
			}
		}

		public override bool IsValidControlValues()
		{
			return true;
		}

		private void SetUpControls()
		{
			Root = new FlowLayoutPanel
			       {
			       	FlowDirection = FlowDirection.TopDown,
			       	AutoScroll = true,
			       	AutoSize = true
			       };

			FlowLayoutPanel contents = new FlowLayoutPanel();
			contents.AutoSize = true;

			Label titleLabel = new Label { Text = Target.GetType().FullName, AutoSize = true };
			Root.Controls.Add( titleLabel );
			Root.Controls.Add( contents );

			PropertyInfo[] infos = Target.GetType().GetProperties( BindingFlags.Instance | BindingFlags.Public );

			foreach ( PropertyInfo info in infos )
			{
				if ( !CanEdit( info ) )
				{
					continue;
				}

				if ( info.PropertyType ==
				     typeof ( string ) )
				{
					LabeledTextBox labeledTextBox = CreateLabeledTextBox( info );

					contents.Controls.Add( labeledTextBox );
				}
				else if ( info.PropertyType ==
				          typeof ( bool ) )
				{
					CheckBox checkBox = CreateCheckBox( info );

					contents.Controls.Add( checkBox );
				}
			}
		}

		private bool CanEdit( PropertyInfo info )
		{
			return info.CanRead && info.CanWrite;
		}

		private LabeledTextBox CreateLabeledTextBox( PropertyInfo info )
		{
			LabeledTextBox labeledTextBox = new LabeledTextBox { TitleText = info.Name };

			labeledTextBox.Width = 300;

			Property targetProperty = new Property { PropertyInfo = info, Target = Target };

			Property controlProperty = new Property { Target = labeledTextBox.TextBox };
			Type controlType = labeledTextBox.TextBox.GetType();
			controlProperty.PropertyInfo = controlType.GetProperty( "Text" );

			PropertyBindings.Add( targetProperty, controlProperty );
			return labeledTextBox;
		}

		private CheckBox CreateCheckBox( PropertyInfo info )
		{
			CheckBox checkBox = new CheckBox { Text = info.Name };
			checkBox.AutoSize = true;
			Property targetProperty = new Property { PropertyInfo = info, Target = Target };

			Property controlProperty = new Property { Target = checkBox };
			Type controlType = typeof ( CheckBox );
			controlProperty.PropertyInfo = controlType.GetProperty( "Checked" );

			PropertyBindings.Add( targetProperty, controlProperty );
			return checkBox;
		}
	}
}